"
COPYRIGHT (c) 2020 LabWare
"
"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

Announcer subclass:#RGEnvironmentAnnouncer
	instanceVariableNames:'suspended private storedAnnouncements'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Announcements'
!

RGEnvironmentAnnouncer class instanceVariableNames:'announcer'

"
 No other class instance variables are inherited by this class.
"
!

RGEnvironmentAnnouncer comment:'This class plays the role of the announcer for events raised by the Pharo system.

BEWARE: You should not need to subclass me. Think about just using me to send your announces (if you need system behavior) or using your own announcers as instance or class  variables.'
!

!RGEnvironmentAnnouncer class methodsFor:'documentation'!

copyright
"
COPYRIGHT (c) 2020 LabWare


"
! !

!RGEnvironmentAnnouncer methodsFor:'accessing'!

private
    ^private ifNil: [ private := Announcer new ]
! !

!RGEnvironmentAnnouncer methodsFor:'announce'!

announce: anAnnouncement
    self isSuspended 
        ifFalse: [ 
            self private announce: anAnnouncement.
            super announce: anAnnouncement ]
        ifTrue:[
            storedAnnouncements ifNotNil:[ storedAnnouncements add: anAnnouncement ]
        ].
!

suspendAllWhile: aBlock
    | oldSuspended |
    oldSuspended := self isSuspended.
    suspended := true.
    ^aBlock ensure: [ suspended := oldSuspended ]
!

suspendAllWhileStoring: aBlock
    | reentring |
    " Suspend all the announcements, storing them in an OrderedCollection, then returns this collection"
    
    reentring := storedAnnouncements isNotNil.
    
    reentring ifFalse:[
        storedAnnouncements := OrderedCollection new.
    ].

    [ 
        self suspendAllWhile: aBlock.
        ^ storedAnnouncements.
    ] ensure:[
        reentring ifFalse:[
            storedAnnouncements := nil.
        ]
    ]
! !

!RGEnvironmentAnnouncer methodsFor:'error handling'!

behaviorAdded: anRGBehavior

    self announce: (ClassAdded class: anRGBehavior category: nil)
!

behaviorCommentModified: anRGBehavior

    self announce: (ClassCommented classCommented: anRGBehavior)
!

behaviorDefinitionChangedFrom: oldRGBehavior to: newRGBehavior

    self announce: (ClassModifiedClassDefinition  
        classDefinitionChangedFrom:  oldRGBehavior
        to: newRGBehavior)
!

behaviorDefinitionModified: anRGBehavior

    self announce: (ClassModificationApplied toClass: anRGBehavior)
!

behaviorModificationAppliedTo: anRGBehavior

    self announce: (ClassModificationApplied toClass: anRGBehavior)
!

behaviorParentRenamed: anRGBehavior from: oldName

    self announce: (ClassParentRenamed 
        classParentOf: anRGBehavior
        renamedFrom: oldName
        to: anRGBehavior name)
!

behaviorRecategorized: anRGBehavior 

    self announce: (ClassRecategorized 
        class: anRGBehavior 
        recategorizedFrom: nil 
        to: anRGBehavior category)
!

behaviorRemoved: anRGBehavior

    self announce: (ClassRemoved 
        class: anRGBehavior category: anRGBehavior category)
!

behaviorRenamed: anRGBehavior from: oldName

    self announce: (ClassRenamed 
        class: anRGBehavior 
        category: anRGBehavior category 
        oldName: oldName 
        newName: anRGBehavior name)
!

behaviorReorganized: anRGBehavior

    self announce: (ClassReorganized class: anRGBehavior)
!

methodAdded: aMethod 

    self announce: (MethodAdded method: aMethod)
!

methodRemoved: aMethod 

    self announce: (MethodRemoved methodRemoved: aMethod protocol: aMethod protocol origin: aMethod parent)
! !

!RGEnvironmentAnnouncer methodsFor:'subscription'!

unsubscribe: anObject
    self private unsubscribe: anObject.	
    super unsubscribe: anObject.
! !

!RGEnvironmentAnnouncer methodsFor:'testing'!

isSuspended
    ^suspended ifNil: [ suspended := false ]
! !

