"{ Package: 'stx:goodies/ring/core' }"

"{ NameSpace: Smalltalk }"

Object subclass:#RGEnvironmentQueryInterface
	instanceVariableNames:'environment behaviorsDictionary'
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Core-Environment'
!

RGEnvironmentQueryInterface comment:''
!

!RGEnvironmentQueryInterface class methodsFor:'as yet unclassified'!

for: anRGEnvironment

    ^ self new 
        environment: anRGEnvironment;
        yourself.
! !

!RGEnvironmentQueryInterface methodsFor:'accessing'!

allClasses

    ^ self behaviors select: [ :each | each isClass ] 
!

behaviorNamed: aSymbol

    ^ self behaviorNamed: aSymbol ifAbsent: [ nil]
!

environment
    ^ environment
!

environment: anObject
    environment := anObject
! !

!RGEnvironmentQueryInterface methodsFor:'as yet unclassified'!

allClassesAndTraits

    ^ self behaviors select: [ :each | each isClass or: [ each isTrait and: [ each isMetaclassTrait not ] ] ]
!

behaviorNamed: aSymbol ifAbsent: aBlock

    | found siblingName | 
    
    found := self behaviorNamedExactlyAs: aSymbol ifAbsent: [nil].
    found ifNotNil: [ ^ found ].
    
    (aSymbol endsWith: ' class') 
        ifTrue: [ 
            siblingName := (aSymbol withoutSuffix: ' class').
            found := self behaviorNamedExactlyAs: siblingName ifAbsent: [nil].
            found ifNotNil: [ 
                found isClass ifTrue: [ ^ found metaclass ]]].

    (aSymbol endsWith: ' classTrait') 
        ifTrue: [ 
            siblingName := (aSymbol withoutSuffix: ' classTrait').
            found := self behaviorNamedExactlyAs: siblingName ifAbsent: [nil].
            found ifNotNil: [ 
                found isTrait ifTrue: [ ^ found classTrait ]]].
        
    siblingName := (aSymbol, ' class').
    found := self behaviorNamedExactlyAs: siblingName ifAbsent: [nil].
    found ifNotNil: [ ^ found baseClass ].		

    siblingName := (aSymbol, ' classTrait').
    found := self behaviorNamedExactlyAs: siblingName ifAbsent: [nil].
    found ifNotNil: [ ^ found baseTrait ].		
                    
    ^ aBlock value.
!

behaviorNamedExactlyAs: aSymbol 

    ^ self behaviorNamedExactlyAs: aSymbol ifAbsent: [nil]
!

behaviorNamedExactlyAs: aSymbol ifAbsent: aBlock

    ^ self behaviorsDictionary at: aSymbol asSymbol ifAbsent: aBlock
    "	
    self behaviorsDo: [:each |
        (each name = aSymbol) ifTrue: [ ^ each ] ].
    
    ^ nil
    "
!

behaviors
    "Retrieves the traits defined in the receiver"

    | allBehaviors |
    allBehaviors := IdentitySet new.
    self behaviorsDo: [ :each | allBehaviors add: each].
    ^ allBehaviors asArray
!

behaviorsDo: aBlock

    self environment behaviorsDo: aBlock
!

classNamed: aSymbol

    ^ self behaviorNamed: aSymbol
!

classOrTraitNamed: aSymbol
    
    ^ self behaviorNamed: aSymbol
!

globalVariables
    "Retrieves the traits defined in the receiver"

    | allGlobalVariables |
    allGlobalVariables := IdentitySet new.
    self globalVariablesDo: [ :each | allGlobalVariables add: each].
    ^ allGlobalVariables asArray
!

globalVariablesDo: aBlock

    self environment globalVariablesDo: aBlock
!

includesClassNamed: aSymbol 

    self globalVariablesDo: [ :var |
        var name = aSymbol ifTrue: [ ^ true ] ].

    ^ self behaviorsDictionary 
        at: aSymbol asSymbol 
        ifPresent: [ :element | true ]
        ifAbsent: [ false ]
    "	
    self behaviorsDo: [:each |
        (each name = aSymbol) ifTrue: [ ^ each ] ].
    
    ^ nil
    "
!

packageNamed: aString
    
    self packagesDo: [:each |
        (each name = aString) ifTrue: [ ^ each ] ].
    
    ^ nil
!

packages
    "Retrieves the traits defined in the receiver"

    | allPackages |
    allPackages := IdentitySet new.
    self packagesDo: [ :each | allPackages add: each].
    ^ allPackages asArray
!

packagesDo: aBlock

    self environment packagesDo: aBlock
!

traitNamed: traitName
    "Retrieves an RGTrait object. The traitName could be theMetaClass name"
    | trait |
    
    ^(trait:= self classOrTraitNamed: traitName) isTrait
        ifTrue:[ trait ]
        ifFalse:[ nil ]
! !

!RGEnvironmentQueryInterface methodsFor:'caching'!

behaviorsDictionary

    ^ behaviorsDictionary ifNil: [ 
        behaviorsDictionary := IdentityDictionary new.
        self behaviorsDo: [:each |
            behaviorsDictionary at: each name put: each ]. 
        behaviorsDictionary
        ]
!

invalidateName: aSymbol

    behaviorsDictionary removeKey: aSymbol
!

replaceName: aSymbol with: aNewSymbol

    | anRGBehavior |
    anRGBehavior := behaviorsDictionary at: aSymbol ifAbsent: [ nil ].
    anRGBehavior ifNotNil: [ 
        behaviorsDictionary removeKey: aSymbol.
        behaviorsDictionary at: aNewSymbol put: anRGBehavior.
         ]
    
!

resetBehaviorsDictionary

    behaviorsDictionary := IdentityDictionary new
! !

