"{ Package: 'stx:goodies/ring/core/tests' }"

"{ NameSpace: Smalltalk }"

RGTest subclass:#RGClassDescripitonStrategyTest
	instanceVariableNames:''
	classVariableNames:''
	poolDictionaries:''
	category:'Ring-Tests-Core'
!

RGClassDescripitonStrategyTest comment:''
!

!RGClassDescripitonStrategyTest methodsFor:'tests'!

checkBasicPropertiesOf: anRGBehavior

    self assert: anRGBehavior isBehavior.	
    self assert: anRGBehavior behaviorStrategy isClassStrategy.
    self assert: anRGBehavior behaviorStrategy owner equals: anRGBehavior.
    self assert: anRGBehavior layout isRingResolved not.
    self assert: anRGBehavior layout isFixedLayout.

    
!

testAllSlots

    | env aSuperclass aSubclass |
    
    env := RGEnvironment new.
    aSuperclass := env ensureClassNamed: 'Superclass'.
    aSubclass := env ensureClassNamed: 'Subclass'.
    aSubclass superclass: aSuperclass.

    aSuperclass instanceVariables: #(a b c).
    aSubclass instanceVariables: #(d e f).
    
    self assert: (aSubclass allSlots collect: [:each | each name]) equals: #(a b c d e f).
!

testClassDescriptionMakeResolved

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    self deny: (anRGBehavior hasResolved: #layout).
    anRGBehavior makeResolved.
    self assert: (anRGBehavior hasResolved: #layout).
    
    
!

testClassDescriptionResolvableProperties

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    self assert: (anRGBehavior pvtResolvableProperties asDictionary includesKey: #layout)
!

testCopyForBehaviorDefinitionPostCopy
    | anRGBehavior copy |
    anRGBehavior := RGBehavior newClass.
    copy := anRGBehavior copyForBehaviorDefinition.
    self deny: anRGBehavior behaviorStrategy identicalTo: copy behaviorStrategy.
    self deny: (anRGBehavior behaviorStrategy instVarNamed: #layout) identicalTo: (copy behaviorStrategy instVarNamed: #layout)
    "TODO: check this failing case:"
    "	self deny: anRGBehavior behaviorStrategy layout == copy behaviorStrategy layout."
!

testCreationByMethod

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self checkBasicPropertiesOf: anRGBehavior
!

testCreationByNamed

    | anRGBehavior |
    
    anRGBehavior := RGClass named: #SomeClass.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: anRGBehavior name equals: #SomeClass.
!

testCreationByNamedParent

    | anRGBehavior parent |
    
    parent := RGEnvironment	new.
    anRGBehavior := RGClass named: #SomeClass parent: parent.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: anRGBehavior name equals: #SomeClass.
    self assert: anRGBehavior parent equals: parent.
!

testCreationByParent

    | anRGBehavior parent |
    
    parent := RGEnvironment	new.
    anRGBehavior := RGClass parent: parent.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: (anRGBehavior hasResolvedName) not.
    self assert: anRGBehavior parent equals: parent.
!

testCreationByUnnamed

    | anRGBehavior parent |
    
    anRGBehavior := RGClass parent: parent.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: (anRGBehavior hasResolvedName) not.
!

testCreationByUnresolved

    | anRGBehavior |
    
    anRGBehavior := RGClass unresolved.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: (anRGBehavior hasResolvedName) not.
    self assert: anRGBehavior isRingResolved not.
!

testCreationByUnresolvedNamed

    | anRGBehavior parent |
    
    parent := RGEnvironment	new.
    anRGBehavior := RGClass unresolvedNamed: #SomeClass withParent: parent.
    self checkBasicPropertiesOf: anRGBehavior.
    self assert: anRGBehavior name equals: #SomeClass.
    self assert: anRGBehavior parent equals: parent.
    self assert: anRGBehavior isRingResolved not.
    
!

testDefaultLayout

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: anRGBehavior defaultLayout isPointerLayout 
!

testDefaultMetaclass

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: anRGBehavior defaultMetaClass isMeta 
!

testEmtpyClassVariableString

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: anRGBehavior classVariablesString equals: ''.
!

testInstSize

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    anRGBehavior instanceVariables: #(a b c).
    
    self assert: anRGBehavior instSize equals: 3.
!

testInstanceVariablesString

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    anRGBehavior instanceVariables: #(a b c).
    
    self assert: anRGBehavior instanceVariablesString equals: 'a b c'.
!

testKindOfSubclass

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    self assert: anRGBehavior kindOfSubclass equals: ' subclass: '.
!

testLayout

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: anRGBehavior layout isPointerLayout 
!

testLayoutSet

    | anRGBehavior layout |
    
    anRGBehavior := RGBehavior newClass.
    layout := RGCompiledMethodLayout parent: anRGBehavior.
    anRGBehavior layout: layout.
    self assert: anRGBehavior layout equals: layout.
!

testSetInstanceVariables

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    anRGBehavior instanceVariables: #(a b c).
    
    self assert: anRGBehavior instVarNames equals: #(a b c).
!

testSharedPools

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    
    self assert: anRGBehavior sharedPools isEmpty.
!

testUnresolvedClassGroupCreatorProperties

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: (anRGBehavior superclass propertyNamed: #creator) equals: anRGBehavior.
    self assert: (anRGBehavior metaclass propertyNamed: #creator) equals: anRGBehavior.
    self assert: (anRGBehavior metaclass superclass propertyNamed: #creator) equals: anRGBehavior.
!

testUnresolvedClassGroupRoleProperties

    | anRGBehavior |
    
    anRGBehavior := RGBehavior newClass.
    self assert: (anRGBehavior superclass propertyNamed: #role) equals: #superclass.
    self assert: (anRGBehavior metaclass propertyNamed: #role) equals: #metaclass.
    self assert: (anRGBehavior metaclass superclass propertyNamed: #role) equals: #superclassMetaclass.
! !

